/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2007  SEGGER Microcontroller Systeme GmbH         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.40c                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------

File   : RTOSInit.c
Purpose: Initializes and handles the hardware for embOS as far
         as required by embOS.
         Feel free to modify this file acc. to your
         target system.
-------- END-OF-HEADER -----------------------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/

#ifndef   OS_FSYS
  #define OS_FSYS 50000000
#endif

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ 1000
#endif

#define TIMER_PRESCALE  1          /* prescaler for timer frequency */

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*       and supports UART0 and UART1
*/

#ifndef   OS_UART
  #define OS_UART -1
#endif    

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400
#endif

#define OS_UART_USED ((OS_UART >= 0) && (OS_UART <= 1))

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/

#define IRQ_BASE_ADDR   0xa000000
#define IRQ_STATUS    *(volatile OS_U32*)(IRQ_BASE_ADDR+0x0000)
#define IRQ_RAWSTATUS *(volatile OS_U32*)(IRQ_BASE_ADDR+0x0004)
#define IRQ_ENABLE    *(volatile OS_U32*)(IRQ_BASE_ADDR+0x0008)
#define IRQ_DISABLE   *(volatile OS_U32*)(IRQ_BASE_ADDR+0x000C)

#define TIMER1_BASE_ADDR   0xa800000
#define TIMER1_LOAD   *(volatile OS_U32*)(TIMER1_BASE_ADDR+0x0000)
#define TIMER1_VALUE  *(volatile OS_U32*)(TIMER1_BASE_ADDR+0x0004)
#define TIMER1_CTL    *(volatile OS_U32*)(TIMER1_BASE_ADDR+0x0008)
#define TIMER1_CLEAR  *(volatile OS_U32*)(TIMER1_BASE_ADDR+0x000C)

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for embOS to run.
*       May be modified, if an other timer should be used
*/

void OS_InitHW(void) {
  #define TIMER_RELOAD       OS_FSYS/TIMER_PRESCALE/OS_TICK_FREQ
  IRQ_DISABLE  = 0x0000001F;    /* Disable all interrupts */
  TIMER1_LOAD  = TIMER_RELOAD;
  TIMER1_CTL   = (1 << 7)       /* Enable timer1 */
                 | (1 << 6)     /* Periodic mode */
                 | (0 << 3)     /* prescaler = 1 */
                 | (0 << 2);    /* prescaler = 1 */
  IRQ_ENABLE   = 0x12;          /* enable timer1 interrupt */
  TIMER1_CLEAR = 1;             /* start timer */
  OS_COM_Init();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) { /* Idle loop: No task is ready to exec */
  while (1) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via embOSView or high 
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/

#define HW_TIMER TIMER1_VALUE
#define HW_TIMER_INTERRUPT_PENDING() (IRQ_STATUS & 0x012)

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = TIMER_RELOAD - HW_TIMER;
  OS_U32 time = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = TIMER_RELOAD - HW_TIMER;
    time++;
  }
  return (OS_FSYS/TIMER_PRESCALE/OS_TICK_FREQ)*time  + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= TIMER_PRESCALE;               /* eliminate effect of prescaler */
  return Cycles/(OS_FSYS/1000000);
}


/*********************************************************************
*
*       Communication for embOSView not selected
*
**********************************************************************
*/

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);
  OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
}

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call correspondig service routine.
*       OS_irq_handler is called from embOS OS_IRQ_SERVICE function
*       found in RTOSVect.asm
*/
void OS_irq_handler(void) {
  int pnd;
  OS_EnterInterrupt();
  pnd = IRQ_STATUS;
  OS_USER_irq_func();
  if (pnd & 0x12) {
    TIMER1_CLEAR = 1;
    OS_TickHandler();           /* embOS timer tick handler */
  }
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       Final check of configuration
*
**********************************************************************
*/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*****  EOF  ********************************************************/

