;**********************************************************************
;*                SEGGER MICROCONTROLLER SYSTEME GmbH                 *
;*        Solutions for real time microcontroller applications        *
;**********************************************************************
;*                                                                    *
;*           (C) 2002    SEGGER Microcontroller Systeme GmbH          *
;*                                                                    *
;*        Internet: www.segger.com    Support:  support@segger.com    *
;*                                                                    *
;**********************************************************************
;
;----------------------------------------------------------------------
;File        : init_AT91M40xxx_RAM.s
;Purpose     : startup file for use with external RAM
;---------------------------END-OF-HEADER------------------------------
;

;
; This module performs ROM/RAM remapping (if required), initializes stack 
; pointers and interrupts for each mode, and finally branches to __main in 
; the C library (which eventually calls main()).
;
; On reset, the ARM core starts up in Supervisor (SVC) mode, in ARM state, 
; with IRQ and FIQ disabled.



;*********************************************************************
;*
;*    Configurable values
;*
;*********************************************************************


; with use to copy code from ROM to RAM
ROM_BASE_ADDR            EQU     0x1000000
ROM_SIZE                 EQU     0x0020000
RAM_BASE_ADDR            EQU     0x0030000

; --- Amount of memory (in bytes) allocated for stacks

Len_FIQ_Stack    EQU     0
Len_IRQ_Stack    EQU     256
Len_ABT_Stack    EQU     0
Len_UND_Stack    EQU     0
Len_SVC_Stack    EQU     1024
Len_USR_Stack    EQU     0



;*********************************************************************
;*
;*    Fixed values
;*
;*********************************************************************

; --- Standard definitions of mode bits and interrupt (I & F) flags in PSRs

Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UND        EQU     0x1B
Mode_SYS        EQU     0x1F ; available on ARM Arch 4 and later

I_Bit           EQU     0x80 ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40 ; when F bit is set, FIQ is disabled

; Add lengths >0 for FIQ_Stack, ABT_Stack, UND_Stack if you need them.
; Offsets will be loaded as immediate values.
; Offsets must be 8 byte aligned.

Offset_FIQ_Stack         EQU     0
Offset_IRQ_Stack         EQU     Offset_FIQ_Stack + Len_FIQ_Stack
Offset_ABT_Stack         EQU     Offset_IRQ_Stack + Len_IRQ_Stack
Offset_UND_Stack         EQU     Offset_ABT_Stack + Len_ABT_Stack
Offset_SVC_Stack         EQU     Offset_UND_Stack + Len_UND_Stack
Offset_USR_Stack         EQU     Offset_SVC_Stack + Len_SVC_Stack
;


;*********************************************************************
;*
;*    Reset handler
;*
;*********************************************************************

        AREA    Init, CODE, READONLY
        CODE32
        PRESERVE8        

        IMPORT __use_no_semihosting_swi   ; --- ensure no functions that use semihosting SWIs are linked in from the C library
        IMPORT OS_InitPLL
       
        ENTRY

        EXPORT  Reset_Handler
Reset_Handler
; Remap
        mov     r10, pc                 ; 
        add     r10, r10, #(EBI_init_table-boot_1)      ; points to init-Table
boot_1
        ; Load the address where to jump (in ROM)
        mov     r11, #ROM_BASE_ADDR     ; final target base address (depends on physical ROM address after remap !)
        mov     r12, pc                 ; To be position independent, calculate address offset
        add     r12, r12, #(after_remap-boot_2)    
boot_2                 
        mov     r12, r12, LSL #12       ; mask the top 12 bits
        mov     r12, r12, LSR #12       ; mask the top 12 bits
        add     r12, r12, r11           ; calculate final absolute address in ROM!!!

        
        ; Copy chip select register image to memory controller and command remap
        ldmia   r10!, {r0-r9,r11}       ; load the complete image and the EBI base
        stmia   r11!, {r0-r9}           ; store the complete image with the remap command

        ; jump to ROM at its new address
        ; this instruction was loaded into the pipeline before the remap was done
        mov     pc, r12                 ; jump and break the pipeline

        ; Put constant table here.
    LTORG
EBI_init_table 
  		DCD   ROM_BASE_ADDR+0x2539            ; CS0: Flash at 0x01000000, 16MB, 16 bits, very slow !
	    DCD   0x10000000                      ; CS1: unused
	    DCD   0x20000000                      ; CS2: unused
	    DCD   0x30000000                      ; CS3: unused
	    DCD   0x40000000                      ; CS4: unused
	    DCD   0x50000000                      ; CS5: unused              
	    DCD   0x60000000                      ; CS6: unused
	    DCD   0x70000000                      ; CS7: unused
	    DCD   0x00000001                      ; REMAP command
	    DCD   0x00000010                      ; early read
	    DCD   0xFFE00000                      ; EBI Base address

after_remap

; RAM is now at 0x0.
; Arrive here after remap (Codes still runs in ROM)
; The exception vectors (in vectors.s) must be copied from ROM to the RAM
; The copying is done later by the C library code inside __main


; --- Initialize stack pointer registers

; Enter each mode in turn and set up the stack pointer
        IMPORT  top_of_stacks       ; defined in stack.s and located by scatter file
        LDR     r0, =top_of_stacks

        MSR     CPSR_c, #Mode_FIQ:OR:I_Bit:OR:F_Bit ; No interrupts
        SUB     sp, r0, #Offset_FIQ_Stack


        MSR     CPSR_c, #Mode_ABT:OR:I_Bit:OR:F_Bit ; No interrupts
        SUB     sp, r0, #Offset_ABT_Stack

        MSR     CPSR_c, #Mode_UND:OR:I_Bit:OR:F_Bit ; No interrupts
        SUB     sp, r0, #Offset_UND_Stack

        MSR     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit ; No interrupts
        SUB     sp, r0, #Offset_IRQ_Stack

        MSR     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit ; No interrupts        
        SUB     sp, r0, #Offset_SVC_Stack


; --- Initialize critical IO devices
        ; ...
;        ldr    R0,  =OS_InitPLL
;        ldr    r1, =ROM_BASE_ADDR - RAM_BASE_ADDR
;        add    r0, r0, r1
;        blx    R0     ; Instruction not available on all CPUs
;         mov LR,PC
;         BX R0
; --- Copy ROM to RAM (because it is faster)
; Copy application from ROM to RAM
;       r0 = (int*) pSrc  
;       r1 = (int*) pDest ; Has to be defined as ROMSTART in link file
;       r2 = len (32 bit words!)
        mov         r0, #ROM_BASE_ADDR
        mov         r1, #RAM_BASE_ADDR
        mov         r2, #ROM_SIZE / 4

copy_loop
        ldr       r3, [r0], #+4
        str       r3, [r1], #+4
        subs      r2,  r2,  #+0x1
        bne       copy_loop

; Jump into RAM
        ldr     r12, =after_copy        ; target address was calculated by linker
        mov     pc, r12                 ; jump to new location in RAM

after_copy                              ; This code runs in RAM
        IMPORT  __main
; --- Now enter the C code
        B       __main                  ; Note: Use B not BL, because an application will never return this way

;*********************************************************************
;*
;*      Stack info for embOS
;*
;*********************************************************************


        EXPORT OS_GetStackInfo
OS_GetStackInfo
        IMPORT SVC_Stack_Base
        IMPORT SVC_Stack_Limit
        IMPORT IRQ_Stack_Base
        IMPORT IRQ_Stack_Limit

        IMPORT  top_of_stacks       ; defined in stack.s and located by scatter file
        LDR     r0, =top_of_stacks
; Get the IRQ Stack information
        LDR     r1, =IRQ_Stack_Limit
        sub     r2, r0, #Offset_IRQ_Stack
        STR     r2, [r1]
        LDR     r1, =IRQ_Stack_Base
        SUB     r2, r2, #Len_IRQ_Stack
        STR     r2, [r1]
; Get the Supervisor Stack information
        LDR     r1, =SVC_Stack_Limit
        sub     r2, r0, #Offset_SVC_Stack
        STR     r2, [r1]
        LDR     r1, =SVC_Stack_Base
        SUB     r2, r2, #Len_SVC_Stack
        STR     r2, [r1]
        BX      lr
        END

