;*********************************************************************
;*                SEGGER MICROCONTROLLER SYSTEME GmbH                *
;*        Solutions for real time microcontroller applications       *
;*********************************************************************
;*                                                                   *
;*      (C) 2006    SEGGER Microcontroller Systeme GmbH              *
;*                  www.segger.com                                   *
;*                                                                   *
;*********************************************************************
;
;---------------------------------------------------------------------
;File   : init_AT91SAM7S.s
;Purpose: startup file for AT91SAM7S / AT91SAM7A3 CPUs and embOS
;-------- END-OF-HEADER ----------------------------------------------
;

; This module performs ROM/RAM remapping (if required), initializes stack 
; pointers and interrupts for each mode, and finally branches to __main in 
; the C library (which eventually calls main()).
;
; On reset, the ARM core starts up in Supervisor (SVC) mode, in ARM state, 
; with IRQ and FIQ disabled.


        AREA    Init, CODE, READONLY
        PRESERVE8
; --- ensure no functions that use semihosting SWIs are linked in from the C library
                IMPORT __use_no_semihosting_swi
      
; --- Standard definitions of mode bits and interrupt (I & F) flags in PSRs

Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UND        EQU     0x1B

I_Bit           EQU     0x80 ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40 ; when F bit is set, FIQ is disabled

; ********************************************************************
;
;       Setup
;
; ********************************************************************

; --- Amount of memory (in bytes) allocated for stacks, each length has to be a multiple of 8 !

Len_FIQ_Stack    EQU     0
Len_IRQ_Stack    EQU     256
Len_ABT_Stack    EQU     0
Len_UND_Stack    EQU     0
Len_SVC_Stack    EQU     512
; Remark:  User stack not used; never switch to user mode!


; Add lengths >0 for FIQ_Stack, ABT_Stack, UND_Stack if you need them.
; Offsets will be loaded as immediate values.
; Offsets have to be 8 byte aligned.

Offset_FIQ_Stack         EQU     0
Offset_IRQ_Stack         EQU     Offset_FIQ_Stack + Len_FIQ_Stack
Offset_ABT_Stack         EQU     Offset_IRQ_Stack + Len_IRQ_Stack
Offset_UND_Stack         EQU     Offset_ABT_Stack + Len_ABT_Stack
Offset_SVC_Stack         EQU     Offset_UND_Stack + Len_UND_Stack

; ********************************************************************

        ENTRY

        EXPORT  Reset_Handler
        IMPORT  __main
        IMPORT  __low_level_init
        
Reset_Handler

; --- Initialize stack pointer registers

; Enter each mode in turn and set up the stack pointer
        IMPORT  top_of_stacks       ; Defined in stack.s and located by scatter file
        ldr     r0, =top_of_stacks
        bic     r0, r0, #0x7        ; Ensure, stacks are aligned on an 8 byte boundary
        
        msr     CPSR_c, #Mode_FIQ:OR:I_Bit:OR:F_Bit ; No interrupts
        sub     sp, r0, #Offset_FIQ_Stack

        msr     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit ; No interrupts
        sub     sp, r0, #Offset_IRQ_Stack

        msr     CPSR_c, #Mode_ABT:OR:I_Bit:OR:F_Bit ; No interrupts
        sub     sp, r0, #Offset_ABT_Stack

        msr     CPSR_c, #Mode_UND:OR:I_Bit:OR:F_Bit ; No interrupts
        sub     sp, r0, #Offset_UND_Stack


        msr     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit ; No interrupts        
        sub     sp, r0, #Offset_SVC_Stack

;       Application has to be called in supervisor mode

; --- Initialize critical IO devices
        ldr     R12,=__low_level_init
        mov     LR,PC
        bx      R12
                
        IMPORT  __main

; --- Now enter the C code
        b       __main   ; note use B not BL, because an application will never return this way


; ********************************************************************

;
; Get Stack info routines, needed for embOS stack check
;

        EXPORT OS_GetStackInfo

OS_GetStackInfo
        IMPORT SVC_Stack_Base
        IMPORT SVC_Stack_Limit
        IMPORT IRQ_Stack_Base
        IMPORT IRQ_Stack_Limit

        IMPORT  top_of_stacks       ; Defined in stack.s and located by scatter file
        ldr     r0, =top_of_stacks
        bic     r0, r0, #0x07       ; Ensure stacks are aligned on an 8 byte boundary
; Get the IRQ Stack information
        ldr     r1, =IRQ_Stack_Limit
        sub     r2, r0, #Offset_IRQ_Stack
        str     r2, [r1]
        ldr     r1, =IRQ_Stack_Base
        sub     r2, r2, #Len_IRQ_Stack
        str     r2, [r1]
; Get the Supervisor Stack information
        ldr     r1, =SVC_Stack_Limit
        sub     r2, r0, #Offset_SVC_Stack
        str     r2, [r1]
        ldr     r1, =SVC_Stack_Base
        sub     r2, r2, #Len_SVC_Stack
        str     r2, [r1]
        bx      lr

; ********************************************************************

        END

; ********************************************************************

