/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2007  SEGGER Microcontroller Systeme GmbH         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.40c                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit.c   (for Motorola DragonballMX1 CPU)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*       Configuration
*
**********************************************************************

  Define clock frequency for CPU

  Select UART for embOSView, set baudrate,
  If you do not want (or can not due to hardware limitations) to dedicate
  a UART to embOSView, please define OS_UART to -1
*/


#ifndef OS_SSYS
  #define OS_SSYS 48000000L
#endif

#ifndef OS_PDIV
  #define OS_PDIV 11
#endif

#ifndef OS_PCLK
  #define OS_PCLK OS_SSYS/(OS_PDIV + 1)
#endif

#ifndef OS_TICK_FREQ
  #define OS_TICK_FREQ 1000
#endif

#define TIMER_PRESCALE  1              // prescaler for timer frequency  

#ifndef OS_UART
  #define OS_UART 0
#endif    

#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400
#endif

/****** End of configuration settings *******************************/

#define OS_UART_USED ((OS_UART == 0) || (OS_UART == 1))

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/

/* PLL and Clock Controller Module Register */
#define CSCR   *(volatile OS_U32*)0x0021B000 /* Clock Source Control Register */
#define PCDR   *(volatile OS_U32*)0x0021B020 /* Peripheral Clock Divider Register */
#define MPCTL0 *(volatile OS_U32*)0x0021B004 /* MCU PLL Control Register 0 */
#define MPCTL1 *(volatile OS_U32*)0x0021B008 /* MCU PLL and System Clock Control Register 1 */
#define SPCTL0 *(volatile OS_U32*)0x0021B00C /* System PLL Control Register 0 */
#define SPCTL1 *(volatile OS_U32*)0x0021B010 /* System PLL Control Register 1 */

/* EIM Module Register */
#define CS0U *(volatile OS_U32*)0x00220000 /* Chip Select 0 Upper Control Register */
#define CS0L *(volatile OS_U32*)0x00220004 /* Chip Select 0 Lower Control Register */
#define CS2U *(volatile OS_U32*)0x00220010 /* Chip Select 2 Upper Control Register */
#define CS2L *(volatile OS_U32*)0x00220014 /* Chip Select 2 Lower Control Register */
#define EIM  *(volatile OS_U32*)0x00220030 /* EIM Configuration Register */

/* System Control Module Register */
#define FMCR *(volatile OS_U32*)0x0021B808 /* Function Multiplexing Control Register */
#define GPCR *(volatile OS_U32*)0x0021B80C /* Global Peripheral Control Register */
#define GCCR *(volatile OS_U32*)0x0021B810 /* Global Clock Control Register */

/* AITC Module Register */
#define INTCNTL       *(volatile OS_U32*)0x00223000     /* Interrupt Control Register                 */
#define NIMASK        *(volatile OS_U32*)0x00223004     /* Normal Interrupt Mask Register             */
#define INTENNUM      *(volatile OS_U32*)0x00223008     /* Interrupt Enable Number Register           */
#define INTDISNUM     *(volatile OS_U32*)0x0022300C     /* Interrupt Disable Number Register          */
#define INTENABLEH    *(volatile OS_U32*)0x00223010     /* Interrupt Enable Register High             */
#define INTENABLEL    *(volatile OS_U32*)0x00223014     /* Interrupt Enable Register Low              */
#define INTTYPEH      *(volatile OS_U32*)0x00223018     /* Interrupt Type Register High               */
#define INTTYPEL      *(volatile OS_U32*)0x0022301C     /* Interrupt Type Register Low                */
#define NIPRIORITY7   *(volatile OS_U32*)0x00223020     /* Normal Interrupt Priority Level Register 7 */
#define NIPRIORITY6   *(volatile OS_U32*)0x00223024     /* Normal Interrupt Priority Level Register 6 */
#define NIPRIORITY5   *(volatile OS_U32*)0x00223028     /* Normal Interrupt Priority Level Register 5 */ 
#define NIPRIORITY4   *(volatile OS_U32*)0x0022302C     /* Normal Interrupt Priority Level Register 4 */ 
#define NIPRIORITY3   *(volatile OS_U32*)0x00223030     /* Normal Interrupt Priority Level Register 3 */ 
#define NIPRIORITY2   *(volatile OS_U32*)0x00223034     /* Normal Interrupt Priority Level Register 2 */ 
#define NIPRIORITY1   *(volatile OS_U32*)0x00223038     /* Normal Interrupt Priority Level Register 1 */ 
#define NIPRIORITY0   *(volatile OS_U32*)0x0022303C     /* Normal Interrupt Priority Level Register 0 */ 
#define NIVECSR       *(volatile OS_U32*)0x00223040     /* Normal Interrupt Vector and Status Register*/ 
#define INTSRCH       *(volatile OS_U32*)0x00223048     /* Interrupt Source Register High             */ 
#define INTSRCL       *(volatile OS_U32*)0x0022304C     /* Interrupt Source Register Low              */ 
#define INTFRCH       *(volatile OS_U32*)0x00223050     /* Interrupt Force Register High              */ 
#define INTFRCL       *(volatile OS_U32*)0x00223054     /* Interrupt Force Register Low               */ 
#define NIPNDH        *(volatile OS_U32*)0x00223058     /* Normal Interrupt Pending Register High     */ 
#define NIPNDL        *(volatile OS_U32*)0x0022305C     /* Normal Interrupt Pending Register Low      */ 


/* GP Timers Module Register */
#define TCTL1         *(volatile OS_U32*)0x00202000     /* Timer 1 Control Register   */
#define TPRER1        *(volatile OS_U32*)0x00202004     /* Timer 1 Prescaler Register */
#define TCMP1         *(volatile OS_U32*)0x00202008     /* Timer 1 Compare Register   */
#define TCR1          *(volatile OS_U32*)0x0020200C     /* Timer 1 Capture Register   */
#define TCN1          *(volatile OS_U32*)0x00202010     /* Timer 1 Counter Register   */
#define TSTAT1        *(volatile OS_U32*)0x00202014     /* Timer 1 Status Register    */

/* GPIO Module Register */
#define GIUSA         *(volatile OS_U32*)0x0021c020     /* GPIO A In Use Register       */
#define GPRA          *(volatile OS_U32*)0x0021c038     /* General Purpose Register   */
#define GIUSB         *(volatile OS_U32*)0x0021c120     /* GPIO B In Use Register       */
#define GPRB          *(volatile OS_U32*)0x0021c138     /* General Purpose Register   */
#define GIUSC         *(volatile OS_U32*)0x0021c220     /* GPIO C In Use Register       */
#define GPRC          *(volatile OS_U32*)0x0021c238     /* General Purpose Register   */



#define TIMER1_INT   59

/*********************************************************************
*
*       Communication for embOSView via UART (optional)
*
**********************************************************************
*/

#if OS_UART_USED


#if (OS_UART == 0)         // Assign internal sfr names to UART0 sfrs
  #define UART_RX_INT   30
  #define UART_TX_INT   29
  
  #define UTXD    *(volatile OS_U32*)0x00206040    // UART1 Transmitter Register 1
  #define URXD_1   *(volatile OS_U32*)0x00206000    // UART1 Receiver Register n
  #define URXD_2   *(volatile OS_U32*)0x00206004    // UART1 Receiver Register n
  #define URXD_3   *(volatile OS_U32*)0x00206008    // UART1 Receiver Register n

  #define UCR1    *(volatile OS_U32*)0x00206080    // UART1 Control Register 1
  #define UCR2    *(volatile OS_U32*)0x00206084    // UART1 Control Register 2
  #define UCR3    *(volatile OS_U32*)0x00206088    // UART1 Control Register 3
  #define UCR4    *(volatile OS_U32*)0x0020608C    // UART1 Control Register 4
  #define UFCR    *(volatile OS_U32*)0x00206090    // UART1 FIFO Control Register

  #define USR1    *(volatile OS_U32*)0x00206094    // UART1 Status Register 2
  #define USR2    *(volatile OS_U32*)0x00206098    // UART1 Status Register 2
  #define UBIR    *(volatile OS_U32*)0x002060A4    // UART1 BRM Incremental Register
  #define UBMR    *(volatile OS_U32*)0x002060A8    // UART1 BRM Modulator Register

#endif  

#endif
/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static int _Dummy;

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/
#if OS_UART_USED

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART
*/
#define UART_RX_READY  (1 << 9)

static void OS_ISR_rx(void) {
  OS_U8 RxData, RxError;
  if (USR1 & UART_RX_READY) {
    do {
      OS_U16 RXData16;
      RXData16 = URXD_1;
      RxData  =  RXData16 & 0xff;                // Read data
      RxError = (RXData16 >> 8);            // Read Error condition
      //  Check and read all received data bytes
      if (RxError == (0x80)) {
        OS_OnRx(RxData);
      } else {
        USR1 &= ~(2200);
        USR2 = 0;
      }
    } while (USR1 & UART_RX_READY);
  }
}

/*********************************************************************
*
*       OS_ISR_rt() Tx interrupt handler for UART
*/
#define UART_TX_READY  (1 << 3)

static void OS_ISR_tx(void) {
  if ((USR2 & UART_TX_READY)) { //  Tx Interrupt pending
    if (OS_OnTx()) {
      //  No more bytes sent, disable tx interrupt
      //  Will be re-enabled in OS_COM_Send1()
      INTDISNUM = UART_TX_INT;
    }
  }  
}

#endif

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitPLL
*
* Function description
*   Initialize main clock and PLL
*   Called from startup code; needs to be done as early as possible
*   in order to keep boot time short
*/
void OS_InitPLL(void) {  
  FMCR   = 0x00000000;   /* Disable SDCS0, SDCS1, EBR and SPI */
  GPCR   = 0x000003fb;   /* Set CSI,MMA in privileged         */
                         /* mode access, Driving Strength     */
                         /* Ctrl/Addr/Data Bus = 100MHz 15pF  */
  GCCR   = 0x00000000;   /* Disable all peripheral clock such as USB/MMA/CSI/SPI */
  MPCTL0 = 0x06852db9;   /* Setup CPUCLK to 192 MHz*/
  MPCTL1 = 0x00000000;   /* BRM contains first order */
  SPCTL0 = 0x1e192cef;   /* Setup SYSCLK to 48 MHz*/
  SPCTL1 = 0x00000000;   /* System PLL is not locked*/
  CSCR   = 0x0a300003;   /* Enable CPU/System PLL, restart them */
                         /* USB CLK = SYSCLK */ 
  PCDR   = 0x000b00bb;   /* PerClk1, PerClk2, PerClk3 = SYSClk / 12 */ 
  CS2U   = 0x00000000;   /* 32bit access for RAM, CS2 Signal enabled*/
  CS2L   = 0x00000601;
  CS0U   = 0x00000801;   /* Setup CS0 (Flash) */
  CS0L   = 0x11110d01;
  EIM    = 0x00000000;  /* disable burst clock mode */
}

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for embOS to run.
*       May be modified, if an other timer should be used
*/

#define TIMER_RELOAD OS_PCLK/TIMER_PRESCALE/OS_TICK_FREQ

void OS_InitHW(void) {
  unsigned int *pVector;
  OS_DI();
  GIUSA  &= ~(0 << 1);
  GPRA   &= ~(0 << 1);
  NIMASK  = 0;               // Disable Interrupts with priority 0
  TCTL1   = 0;               // Diable timeer 0
  TPRER1  = (TIMER_PRESCALE-1);  // set prescaler
  TCMP1   = TIMER_RELOAD;    // Set compare register
  TCTL1   = (1 << 0)         // Enable timer 0
          | (1 << 1)         // Set peripheral clock to PERCLK1
          | (1 << 4)         // Enable interrupt on compare
          | (0 << 6)         // disable capture mode
          | (0 << 8);        // restart mode
  NIPRIORITY7 = (1 << 12);   // set priority 1 to Timer 1 interrupt
  INTTYPEH    = 0;           // All interrupts generate a normal interrupt
  INTENNUM    = TIMER1_INT;  // Enable Timer 1 interrupt;
  OS_USEPARA(_Dummy);
  _Dummy   = TSTAT1;
  TSTAT1   = 0;
  OS_COM_Init();
  pVector  = (unsigned int*) 0x18;
  *pVector = 0xE59FF000;            // Place command LDR PC, [PC]
  pVector  = (unsigned int*) 0x20;  // which requires (reads) routine address at 0x20
  *pVector = (unsigned long) &OS_IRQ_SERVICE;
  OS_RestoreI();
}         

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {     // Idle loop: No task is ready to exec
  while (1) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via embOSView or high 
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = TCN1;               // Read current timer value
  OS_U32 time = OS_GetTime32();            // Read current embOS time 
  if ((NIVECSR >> 16) == TIMER1_INT) { // Timer Interrupt pending ?
    t_cnt = TCN1;                         // Adjust result     
    time++;
  }
  return TIMER_RELOAD*time + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= TIMER_PRESCALE;           // eliminate effect of prescaler
  return Cycles/(OS_PCLK/1000000);
}

#if OS_UART_USED

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/

void OS_COM_Send1(OS_U8 c) {
  UTXD = c;
  INTENNUM = UART_TX_INT;
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for embOSView
*/

void OS_COM_Init(void) {         // Initialize UART, enable UART interrupts
  GIUSC &= ~(1 << 12);        // Port multiplexing: use UART0 RX
  GPRC  &= ~(1 << 12);        // Port multiplexing: use UART0 RX
  GIUSC &= ~(1 << 11);        // Port multiplexing: use UART0 TX
  GPRC  &= ~(1 << 11);        // Port multiplexing: use UART0 TX
  NIPRIORITY3 |= (1 << 24)    // set priority 1 to UART 1 RX/TX interrupts
              |  (1 << 20);
  INTENNUM    = UART_RX_INT;  // Enable UART RX interrupt;
  INTENNUM    = UART_TX_INT;  // Enable UART TX interrupt;
  UCR4       = (32 << 10)     // 32 byte FIFO buffer
             | (1 << 6)       // 16MHz reference clock
             | (1 << 0);
  UFCR       = (0x20 << 10)   // 32 byte TX FIFO
             | (0x00 << 7)    // PERCLK / 6
             | (0x01 << 0);   // 32 byte RX FIFO
  UBIR       = 15;            // set baudrate to 38400
  UBMR       = 16;            // formula: (Baudrate*16)/(PERCLK / 6) = (UBIR-1)/(UBMR-1)
  UCR1       = (1 << 13)      // enable TX Ready interrupt
             | (1 << 9)       // enable RX Ready interrupt
             | (1 << 2)       // enable UART clock
             | (1 << 0);      // enable UART
  UCR2       = (1 << 14)      // ignore UART RTS pin
             | (0 << 8)       // if parity: odd parity
             | (1 << 7)       // disable parity
             | (0 << 6)       // 1 stop bit
             | (1 << 5)       // 8bit data
             | (1 << 2)       // enable transmitter
             | (1 << 1)       // enable receiver
             | (1 << 0);      // do not reset UART
}

#else  /* selected UART not supported, using dummies */

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           // avoid compiler warning
  OS_COM_ClearTxActive();  // let embOS know that Tx is not busy
}

#endif /*  OS_UART_USED  */

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call correspondig service routine.
*       OS_irq_handler is called from embOS OS_IRQ_SERVICE function
*       found in RTOSVect.asm
*/

void OS_irq_handler(void) {
  int PendingIRQ;
  OS_EnterInterrupt();
  PendingIRQ = (NIVECSR >> 16);
  if (PendingIRQ == TIMER1_INT)  {
    _Dummy    = TSTAT1;
    TSTAT1   = 0;
    OS_TickHandler();
  } else if (PendingIRQ == UART_TX_INT){
    OS_ISR_tx();
  }  else if (PendingIRQ == UART_RX_INT) {
    OS_ISR_rx();  
  } 
  OS_LeaveInterrupt();
}                               // by usage of OS_EnterInterrupt(); 

/*********************************************************************
*
*       Final check of configuration
*
**********************************************************************
*/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*****  EOF  ********************************************************/

