/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2007  SEGGER Microcontroller Systeme GmbH         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.40c                                         *
*                                                                    *
**********************************************************************

------------- END-OF-HEADER ------------------------------------------
*/

#include "RTOS.H"

#ifdef __ICCARM__
#include "inarm.h"
#endif

/*********************************************************************
*
*       Configuration
*
**********************************************************************

  Define clock frequency for CPU

  Select UART for embOSView, set baudrate,
  If you do not want (or can not due to hardware limitations) to dedicate
  a UART to OSView, please define OS_UART to -1
*/


#ifndef OS_FSYS
  #define OS_FSYS 74000000
#endif

#ifndef OS_TICK_FREQ
  #define OS_TICK_FREQ 1000
#endif

#define TIMER_PRESCALE  1             // prescaler for timer frequency  

#ifndef OS_UART
  #define OS_UART 0
#endif    

#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400
#endif

#ifndef OS_PCLK_TIMER
  #define OS_PCLK_TIMER 512000
#endif

/****** End of configuration settings *******************************/

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/

#define __PBDR          *(volatile OS_U8*)  0x80000001
#define __PBDDR         *(volatile OS_U8*)  0x80000041
#define __SYSCON1       *(volatile OS_U32*) 0x80000100
#define __INTSR1        *(volatile OS_U32*) 0x80000240
#define __INTMR1        *(volatile OS_U32*) 0x80000280
#define __TC1D          *(volatile OS_U16*) 0x80000300
#define __TC2D          *(volatile OS_U16*) 0x80000340
#define __UARTDR1       *(volatile OS_U16*) 0x80000480
#define __UBRLCR1       *(volatile OS_U32*) 0x800004C0
#define __TC1EOI        *(volatile OS_U32*) 0x800006C0
#define __TC2EOI        *(volatile OS_U32*) 0x80000700
#define __SYSFLG2       *(volatile OS_U32*) 0x80001140
#define __INTMR2        *(volatile OS_U32*) 0x80001280
#define __SYSCON3       *(volatile OS_U32*) 0x80002200
#define __PLLW          *(volatile OS_U32*) 0x80002610

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for OS to run.
*       May be modified, if an other timer should be used
*/
#define TIMER_RELOAD    OS_PCLK_TIMER/1000

void OS_InitHW(void) {
  OS_DI();
  /* Disable all IRQs */
  __INTMR1  = 0x0;          
  __INTMR2  = 0x0;
  __SYSCON1 &= ~0x1000;           /* disable LCD */    
  __SYSCON3 = 0x6;                /* Set CPU to 74 MHz  (memory clock: 36 MHz)*/
  __PLLW = 0x28000000;
#ifdef __ICCARM__
  __no_operation();
#else
  __asm { NOP }
#endif
  /* Init Timer to generate Tick interrupt  */
  __INTMR1  &= ~0x100ul;     /* disable TC1 underflow interrupt */
  __SYSCON1 |= 0x20ul;       /* Set clock in 512kHz mode */
  __SYSCON1 &= ~ 0x10ul;     /* free running */
  __TC1D    = TIMER_RELOAD;  /* Load counter */
  __INTMR1 |= 0x100ul;       /* enable TC1 underflow interrupt */
  OS_RestoreI();
  OS_COM_Init();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {     // Idle loop: No task is ready to exec
  while (1) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via OSView or high 
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
#define HW_TIMER  (__TC1D)
#define HW_TIMER_INTERRUPT_PENDING() (__INTSR1 & 0x0100)

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = TIMER_RELOAD - HW_TIMER;
  OS_U32 time = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = (OS_U32)((OS_I16)((OS_I16)0-(OS_I16)HW_TIMER));
    time++;
  }
  return (OS_PCLK_TIMER/TIMER_PRESCALE/OS_TICK_FREQ)*time  + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= TIMER_PRESCALE;               /* eliminate effect of prescaler */
  return (1000*Cycles)/(OS_PCLK_TIMER/1000);
}
/*********************************************************************
*
*       Communication for OSView via UART (optional)
*
**********************************************************************
*/

#define OS_UART_USED (OS_UART==0)
#if OS_UART_USED

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/
  void OS_COM_Init(void) {
    unsigned long ubrlcr_value;
    unsigned char utemp;
    unsigned long ltemp;
    int pll;
    /* Check bit siz of sysflags2. If it is low we're at 74mhz, else 13mhz */
    if (__SYSFLG2 & 0x40) {
      pll = 0;
    } else {
      pll = 1;
    }
    utemp   = __PBDDR;
    utemp   &= ~0x02ul;     /* Set bit one of prallel reg b to out */
    __PBDDR = utemp;
    ubrlcr_value = 0;
#if (OS_BAUDRATE==9600)
    ubrlcr_value |= pll ? 23 : 0x0b;
#elif (OS_BAUDRATE==38400)
    ubrlcr_value |= pll ? 5 : 0x02;
#else
  #error Please specify divisor.
#endif
    ltemp = (unsigned long) (8 - 5);
    ltemp <<= 17;
    ubrlcr_value |= ltemp;
    ltemp = 0;    /* parity==0 */
    ltemp <<= 13;
    ubrlcr_value |= ltemp;
    __UBRLCR1 = ubrlcr_value;
    __INTMR1 |= 0x2000ul;    /* enable RX */
    ltemp     = __SYSCON1;
    ltemp    |= 0x100; // Bit 8 == enable uart 1 
    __SYSCON1 = ltemp;
    __PBDR = 2;
  }

/*********************************************************************
*
*       _OS_ISR_rx()
*       OS rx interrupt handler for communication to OSView
*/
  static void _OS_ISR_rx(void) {
    OS_U16 data;
    data = __UARTDR1;     
    if (!(data&0x0300)) {
      OS_OnRx(data & 0xff);
    }
  }

/*********************************************************************
*
*       _OS_ISR_tx()
*       OS tx interrupt handler for communication to OSView
*/
  static void _OS_ISR_tx(void) {
    __INTMR1 &= ~0x1000ul;    /* disable TX */
    OS_OnTx();
  }

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
  void OS_COM_Send1(OS_U8 c) {
    __INTMR1 |= 0x1000ul;     /* enable TX */
    __UARTDR1 = (OS_U16) c;
  }
  
#else  /* UART handlers not defined, using dummys */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy */
  }
#endif

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call correspondig service routine.
*       OS_irq_handler is called from OS OS_IRQ_SERVICE function
*       found in RTOSVect.asm
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_U32 status;
  OS_EnterInterrupt();
  status  = __INTSR1;
  status &= __INTMR1;
  OS_USER_irq_func();
#if (OS_UART_USED)
  if (status & 0x1000) {
    _OS_ISR_tx();
  }
  if (status & 0x2000) {
    _OS_ISR_rx();
  }
#endif
  if (status & 0x0100) {
    __TC1D    = TIMER_RELOAD;   /* Load counter */      
    __TC1EOI  = 1;              /* Clear interrupt. */
    OS_TickHandler();
  }
  OS_LeaveInterrupt();
}

/*****  EOF  ********************************************************/

