/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2007  SEGGER Microcontroller Systeme GmbH         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.40c                                         *
*                                                                    *
**********************************************************************

-------- END-OF-HEADER -----------------------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/

#ifndef OS_FSYS
  #define OS_FSYS 50000000
#endif

#ifndef OS_TICK_FREQ
  #define OS_TICK_FREQ 1000
#endif

#define TIMER_PRESCALE  1              /* prescaler for timer frequency */

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*       and supports UART0 and UART1
*/

#ifndef OS_UART
  #define OS_UART 0
#endif

#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400
#endif

#define OS_UART_USED ((OS_UART >= 0) && (OS_UART <= 1))

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/

#define __SYSCFG  *(volatile OS_U32*) 0x03ff0000

#define __INTMOD  *(volatile OS_U32*) 0x03ff4000
#define __INTPND  *(volatile OS_U32*) 0x03ff4004
#define __INTMSK  *(volatile OS_U32*) 0x03ff4008

#define __IOPMOD  *(volatile OS_U32*) 0x03ff5000
#define __IOPDATA *(volatile OS_U32*) 0x03ff5008

#define __TMOD    *(volatile OS_U32*) 0x03ff6000
#define __TDATA0  *(volatile OS_U32*) 0x03ff6004
#define __TDATA1  *(volatile OS_U32*) 0x03ff6008
#define __TCNT0   *(volatile OS_U32*) 0x03ff600c
#define __TCNT1   *(volatile OS_U32*) 0x03ff6010

#define __ULCON0  *(volatile OS_U32*) 0x03ffd000
#define __UCON0   *(volatile OS_U32*) 0x03ffd004
#define __USTAT0  *(volatile OS_U32*) 0x03ffd008
#define __UTXBUF0 *(volatile OS_U32*) 0x03ffd00c
#define __URXBUF0 *(volatile OS_U32*) 0x03ffd010
#define __UBRDIV0 *(volatile OS_U32*) 0x03ffd014

#define __ULCON1  *(volatile OS_U32*) 0x03ffe000
#define __UCON1   *(volatile OS_U32*) 0x03ffe004
#define __USTAT1  *(volatile OS_U32*) 0x03ffe008
#define __UTXBUF1 *(volatile OS_U32*) 0x03ffe00c
#define __URXBUF1 *(volatile OS_U32*) 0x03ffe010
#define __UBRDIV1 *(volatile OS_U32*) 0x03ffe014

/*  Definiton of sfrs assigned to UART used for OSView */

#if OS_UART_USED
  #if (OS_UART==1)
    #define TX_PENDING       0x0040
    #define RX_PENDING       0x0080
    #define UART_STATUS      __USTAT1
    #define UART_RXBUF       __URXBUF1
    #define UART_TXBUF       __UTXBUF1
    #define UART_LINECON     __ULCON1
    #define UART_CONTROL     __UCON1
    #define UART_BAUD        __UBRDIV1
  #elif (OS_UART==0)
    #define TX_PENDING       0x0010
    #define RX_PENDING       0x0020
    #define UART_STATUS      __USTAT0
    #define UART_RXBUF       __URXBUF0
    #define UART_TXBUF       __UTXBUF0
    #define UART_LINECON     __ULCON0
    #define UART_CONTROL     __UCON0
    #define UART_BAUD        __UBRDIV0
  #endif
#endif

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if OS_UART_USED  /* UART Tx and Rx handler */

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART
*/

static void OS_ISR_rx(void) {
  if(UART_STATUS & 0x20) {
    OS_OnRx(UART_RXBUF);
  }
}

/*********************************************************************
*
*       OS_ISR_rt() Tx interrupt handler for UART
*/

static void OS_ISR_tx(void) {
  OS_OnTx();
}

#endif /* OS_UART_USED */

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/

void OS_InitHW(void) {
  #define TIMER_RELOAD       OS_FSYS/TIMER_PRESCALE/OS_TICK_FREQ
  #define TIMER_TICK_PND     (1 <<10)
  OS_DI();
  __TMOD   = 0;
  __INTPND = 0x00000000;   /* Clear pending interrupts */
  __INTMOD = 0x00000000;
  __TDATA0 = TIMER_RELOAD;
  __TMOD  |= 0x1;
  __INTMSK &= ~((1 << 21) | TIMER_TICK_PND);
  OS_COM_Init();
  OS_RestoreI();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) { /* Idle loop: No task is ready to exec */
  while (1) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via OSView or high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/

#define HW_TIMER __TCNT0
#define HW_TIMER_INTERRUPT_PENDING() (__INTPND & 0x0400)

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = TIMER_RELOAD - HW_TIMER;
  OS_U32 time = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = TIMER_RELOAD - HW_TIMER;
    time++;
  }
  return (OS_FSYS/TIMER_PRESCALE/OS_TICK_FREQ)*time  + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= TIMER_PRESCALE;               /* eliminate effect of prescaler */
  return Cycles/(OS_FSYS/1000000);
}

#if OS_UART_USED
/*********************************************************************
*
*       Communication for OSView via UART0 or UART1 (optional)
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/

  void OS_COM_Init(void) {
    UART_CONTROL = 0x00;
    UART_LINECON = 0x03;
    UART_CONTROL = 0x0d;
    UART_BAUD = (((OS_FSYS/2) / (16*OS_BAUDRATE)-1) << 4);
    __INTMSK &= ~(TX_PENDING | RX_PENDING);
  }

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/

  void OS_COM_Send1(OS_U8 c) {
    UART_TXBUF = c;
  }

/*********************************************************************
*
*       Communication for OSView not selected
*
**********************************************************************
*/

#else  /* No communication routines required, use dummies  */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
  }
#endif /* OS_UART_USED */

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call correspondig service routine.
*       OS_irq_handler is called from OS_IRQ_SERVICE function
*       found in RTOSVect.asm
*/
OS_INTERWORK void OS_irq_handler(void) {
  int pnd;
  OS_EnterInterrupt();
  pnd = __INTPND;
  OS_USER_irq_func();
  #if OS_UART_USED
  if (pnd & RX_PENDING) {
    __INTPND = RX_PENDING;
    OS_ISR_rx();
  }
  if (pnd & TX_PENDING) {
    __INTPND = TX_PENDING;
    OS_ISR_tx();
  }
  #endif /* OS_UART_USED */
  if (pnd & TIMER_TICK_PND) {
    __INTPND = TIMER_TICK_PND;
    OS_TickHandler();           /* OS timer tick handler */
  }
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       Final check of configuration
*
**********************************************************************
*/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*****  EOF  ********************************************************/

