;;; Copyright ARM Ltd 2001. All rights reserved.
;
; This module performs ROM/RAM remapping (if required), initializes stack 
; pointers and interrupts for each mode, and finally branches to __main in 
; the C library (which eventually calls main()).
;
; On reset, the ARM core starts up in Supervisor (SVC) mode, in ARM state, 
; with IRQ and FIQ disabled.


        AREA    Init, CODE, READONLY
        PRESERVE8        
; --- ensure no functions that use semihosting SWIs are linked in from the C library
                IMPORT __use_no_semihosting_swi
      
; --- Standard definitions of mode bits and interrupt (I & F) flags in PSRs

Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UND        EQU     0x1B
Mode_SYS        EQU     0x1F ; available on ARM Arch 4 and later

I_Bit           EQU     0x80 ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40 ; when F bit is set, FIQ is disabled

; --- Amount of memory (in bytes) allocated for stacks

Len_FIQ_Stack    EQU     0
Len_IRQ_Stack    EQU     256
Len_ABT_Stack    EQU     0
Len_UND_Stack    EQU     0
Len_SVC_Stack    EQU     1024
Len_USR_Stack    EQU     1024

; Add lengths >0 for FIQ_Stack, ABT_Stack, UND_Stack if you need them.
; Offsets will be loaded as immediate values.
; Offsets must be 8 byte aligned.

Offset_FIQ_Stack         EQU     0
Offset_IRQ_Stack         EQU     Offset_FIQ_Stack + Len_FIQ_Stack
Offset_ABT_Stack         EQU     Offset_IRQ_Stack + Len_IRQ_Stack
Offset_UND_Stack         EQU     Offset_ABT_Stack + Len_ABT_Stack
Offset_SVC_Stack         EQU     Offset_UND_Stack + Len_UND_Stack
Offset_USR_Stack         EQU     Offset_SVC_Stack + Len_SVC_Stack



        ENTRY

        EXPORT  Reset_Handler

Reset_Handler
  ldr r0, SYSCFG
  ldr r1, SYSCFG_val
  str r1, [r0]

  ldr lr, =copy_vectors
  mov r0, pc
  add r0, r0, #(MemConfig-boot)      ; points to init-Table

boot
  ldmia r0, {r0-r12}
  stmia r0, {r1-r12}
  mov pc, lr
  
  
copy_vectors
  IMPORT |Image$$FLASH$$Base|
;;; Copy exception vectors into RAM
        ldr       r0,   =|Image$$FLASH$$Base|
        mov       r1,   #0x00000000
        mov       r2,   #0x50 / 4
copy_loop
        ldr       r3, [r0], #+4
        str       r3, [r1], #+4
        subs      r2,  r2,  #+0x1
        bne       copy_loop
        ldr       lr,   =after_config
        mov       pc,   lr
;*****************************************************************************
; * Register settings for the following memory configuration:                 *
; *                                                                           *
; *                     ----------------------                                *
; *                    | ROMCON0 - 512K FLASH | 0x01800000 - 0x0187FFFF       *
; *                    |----------------------|                               *
; *                    | ROMCON2 - 256K SRAM  | 0x00040000 - 0x0007FFFF       *
; *                    |----------------------|                               *
; *                    | ROMCON1 - 256K SRAM  | 0x00000000 - 0x0003FFFF       *
; *                     ----------------------                                *
; *                                                                           *
; *****************************************************************************
  LTORG
MemConfig
  DCD 0x03FF3010 ; &EXTDBWIDTH
  DCD 0x0000003E ;  EXTDBWIDTH
  DCD 0x18860030 ;  ROMCON0
  DCD 0x00400010 ;  ROMCON1
  DCD 0x00801010 ;  ROMCON2
  DCD 0x08018020 ;  ROMCON3
  DCD 0x0A020040 ;  ROMCON4
  DCD 0x0C028040 ;  ROMCON5
  DCD 0x00000000 ;  DRAMCON0
  DCD 0x00000000 ;  DRAMCON1
  DCD 0x00000000 ;  DRAMCON2
  DCD 0x00000000 ;  DRAMCON3
  DCD 0x9C218360 ;  REFEXTCON
SYSCFG
  DCD 0x03FF0000 ; &SYSCFG
SYSCFG_val
  DCD 0x03FFFFA0 ;  SYSCFG

after_config
; --- Initialize stack pointer registers

; Enter each mode in turn and set up the stack pointer

        IMPORT  top_of_stacks       ; defined in stack.s and located by scatter file
        LDR     r0, =top_of_stacks

;       MSR     CPSR_c, #Mode_FIQ:OR:I_Bit:OR:F_Bit ; No interrupts
;       SUB     sp, r0, #Offset_FIQ_Stack


;       MSR     CPSR_c, #Mode_ABT:OR:I_Bit:OR:F_Bit ; No interrupts
;       SUB     sp, r0, #Offset_ABT_Stack

;       MSR     CPSR_c, #Mode_UND:OR:I_Bit:OR:F_Bit ; No interrupts
;       SUB     sp, r0, #Offset_UND_Stack

       MSR     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit ; No interrupts
       SUB     sp, r0, #Offset_IRQ_Stack

       MSR     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit ; No interrupts
       SUB     sp, r0, #Offset_SVC_Stack

; --- Initialize critical IO devices
        ; ...
;        IMPORT OS_InitPLL
;        ldr    R0,  =OS_InitPLL
;        mov LR,PC
;        BX R0

                
        IMPORT  __main

; --- Now enter the C code
        B       __main   ; note use B not BL, because an application will never return this way


;*********************************************************************
;*
;*      Stack info for embOS
;*
;*********************************************************************


        EXPORT OS_GetStackInfo
OS_GetStackInfo
        IMPORT SVC_Stack_Base
        IMPORT SVC_Stack_Limit
        IMPORT IRQ_Stack_Base
        IMPORT IRQ_Stack_Limit

        IMPORT  top_of_stacks       ; defined in stack.s and located by scatter file
        LDR     r0, =top_of_stacks
; Get the IRQ Stack information
        LDR     r1, =IRQ_Stack_Limit
        sub     r2, r0, #Offset_IRQ_Stack
        STR     r2, [r1]
        LDR     r1, =IRQ_Stack_Base
        SUB     r2, r2, #Len_IRQ_Stack
        STR     r2, [r1]
; Get the Supervisor Stack information
        LDR     r1, =SVC_Stack_Limit
        sub     r2, r0, #Offset_SVC_Stack
        STR     r2, [r1]
        LDR     r1, =SVC_Stack_Base
        SUB     r2, r2, #Len_SVC_Stack
        STR     r2, [r1]
        BX      lr

MMUentry
        DCD 0x60000c1e
MMUmask
        DCD 0xffffc000
      


        END


